/*
 *  Copyright 2008 biaoping.yin
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.  
 */
package com.frameworkset.common.poolman.util;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NameNotFoundException;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.frameworkset.spi.BaseApplicationContext;

import com.frameworkset.common.poolman.DBUtil;
import com.frameworkset.common.poolman.NestedSQLException;
import com.frameworkset.common.poolman.PoolManConstants;
import com.frameworkset.common.poolman.interceptor.InterceptorInf;
import com.frameworkset.common.poolman.jndi.ContextUtil;
import com.frameworkset.common.poolman.jndi.DummyContextFactory;
import com.frameworkset.common.poolman.security.DBInfoEncrypt;
import com.frameworkset.common.poolman.security.DESDBPasswordEncrypt;
import com.frameworkset.common.poolman.sql.ColumnMetaData;
import com.frameworkset.common.poolman.sql.ForeignKeyMetaData;
import com.frameworkset.common.poolman.sql.PoolManDataSource;
import com.frameworkset.common.poolman.sql.PrimaryKeyMetaData;
import com.frameworkset.common.poolman.sql.TableMetaData;
import com.frameworkset.commons.dbcp.BasicDataSourceFactory;
import com.frameworkset.commons.pool.impl.GenericObjectPool;
import com.frameworkset.orm.adapter.DB;
import com.frameworkset.orm.adapter.DBFactory;
import com.frameworkset.orm.transaction.JDBCTransaction;
import com.frameworkset.orm.transaction.TXDataSource;
import com.frameworkset.orm.transaction.TransactionManager;
import com.frameworkset.util.StringUtil;

/**
 * JDBCPool is an ObjectPool of JDBC connection objects. It is also a
 * javax.sql.ConnectionEventListener, so it can respond to events fired by the
 * PooledConnection implementation, PoolManConnection.
 */
public class JDBCPool {
	private static Logger log = Logger.getLogger(JDBCPool.class);
	public static String TABLE_TYPE_VIEW = "VIEW";
	public static String TABLE_TYPE_TABLE = "TABLE";
	public static String TABLE_TYPE_ALL = "ALL";
	private DataSource datasource;
	

//	protected JDBCPoolMetaData metadata;

	private boolean deployedDataSource;

	private JDBCPoolMetaData info;

	/**
	 * ʱ
	 */
	private long startTime;

	/**
	 * ֹͣʱ
	 */
	private long stopTime;

	private  Context ctx = null;
	private  Context dummyctx = null;
	private  boolean initcontexted = false;
	
	private static Context buildContext(JDBCPoolMetaData meta)
	{
		Context ctx = null;
		if(meta != null)
		{
			if((meta.getJNDIName() == null || meta.getJNDIName().equals("")) 
					&& (meta.getExternaljndiName() == null || meta.getExternaljndiName().equals("")))
			{
				return null;
			}
		}
		if(meta != null && meta.getJndiclass() != null)
		{
			Hashtable environment = new Hashtable();
			environment.put(Context.INITIAL_CONTEXT_FACTORY, meta.getJndiclass());
			if(meta.getJndiurl() != null)
				environment.put(Context.PROVIDER_URL, meta.getJndiurl());
			if(meta.getJndiuser() != null)
				environment.put(Context.SECURITY_PRINCIPAL
						, meta.getJndiuser());
			if(meta.getJndipassword() != null)
				environment.put(Context.SECURITY_CREDENTIALS, meta.getJndipassword());
			ctx = ContextUtil.finaContext(environment, JDBCPool.class.getClassLoader());
		}
		else
		{
			ctx = ContextUtil.finaContext(null, JDBCPool.class.getClassLoader());
		}
		return ctx;
	}

	private  void initctx(JDBCPoolMetaData meta)
	{
		if(ctx == null)
		{

			ctx = buildContext(meta);
			if(meta.getJNDIName() != null && !meta.getJNDIName().equals(""))
			{
				try {
					this.dummyctx = (new DummyContextFactory()).getInitialContext(null);
				} catch (NamingException e) {
					log.info(e.getMessage());
				}
			}
				
			initcontexted = true;
			
		}
	}

	

	private Hashtable preparedStatementPool;

	private Map tableMetaDatasindexByTablename = new java.util.concurrent.ConcurrentHashMap();

	private Set tableMetaDatas = new TreeSet();
	
	
	public final static InterceptorInf defaultInterceptor = new com.frameworkset.common.poolman.interceptor.DummyInterceptor();
	private InterceptorInf interceptor = defaultInterceptor;
	public final static String defaultInterceptor_s = "com.frameworkset.common.poolman.interceptor.DummyInterceptor";
	
	/**
	 * ӳص
	 */
	private DB dbAdapter;

	private String status = "unknown";

	public TableMetaData getTableMetaData(String tableName) {

		return getTableMetaData(null, tableName);
	}

	public TableMetaData getTableMetaData(Connection con, String tableName) {
		if(this.externalDBName == null)
		{
			TableMetaData table = (TableMetaData) tableMetaDatasindexByTablename
					.get(tableName.toLowerCase());
			if (table == null) {
				// ݿԪûԪݼػƣ̬
				if (this.info.getLoadmetadata().equalsIgnoreCase("false")) {
					synchronized (tableMetaDatasindexByTablename) {
						table = (TableMetaData) tableMetaDatasindexByTablename
								.get(tableName.toLowerCase());
						if (table == null) {
							table = this.getTableMetaDataFromDatabase(con,
									tableName);
						}
					}
				}
			}
			return table;
		}
		else
		{
			return SQLManager.getInstance().getPool(externalDBName).getTableMetaData(con, tableName);
		}
	}

	public ColumnMetaData getColumnMetaData(Connection con, String tableName,
			String columnName) {
		TableMetaData table = getTableMetaData(con, tableName);
		if (table != null) {
			// TableMetaData table = getTableMetaData(tableName);
			if (columnName == null || columnName.equals("")) {
				return null;
			}
			return table.getColumnMetaData(columnName);
		}
		return null;
	}

	public ColumnMetaData getColumnMetaData(String tableName, String columnName) {

		return getColumnMetaData(null, tableName, columnName);
	}

	public Set getColumnMetaDatas(String tableName) {

		return getColumnMetaDatas(null, tableName);
	}

	public Set getColumnMetaDatas(Connection con, String tableName) {
		TableMetaData table = getTableMetaData(con, tableName);
		if (table != null)
			return table.getColumns();
		return null;
	}

	public ForeignKeyMetaData getForeignKeyMetaData(String tableName,
			String columnName) {

		return getForeignKeyMetaData(null, tableName, columnName);
	}

	public ForeignKeyMetaData getForeignKeyMetaData(Connection con,
			String tableName, String columnName) {
		TableMetaData table = getTableMetaData(con, tableName);
		if (table != null)
			return table.getForeignKeyMetaData(columnName);
		return null;
	}

	public Set getForeignKeyMetaDatas(String tableName) {

		return getForeignKeyMetaDatas(null, tableName);
	}

	public Set getForeignKeyMetaDatas(Connection con, String tableName) {
		TableMetaData table = getTableMetaData(con, tableName);
		if (table != null)
			return table.getForeignKeys();
		return null;
	}

	public PrimaryKeyMetaData getPrimaryKeyMetaData(String tableName,
			String columnName) {

		return getPrimaryKeyMetaData(null, tableName, columnName);
	}

	public PrimaryKeyMetaData getPrimaryKeyMetaData(Connection con,
			String tableName, String columnName) {
		TableMetaData table = getTableMetaData(con, tableName);
		if (table != null)
			return table.getPrimaryKeyMetaData(columnName);
		return null;
	}

	public Set getPrimaryKeyMetaDatas(String tableName) {
		return getPrimaryKeyMetaDatas(null, tableName);
	}

	public Set getPrimaryKeyMetaDatas(Connection con, String tableName) {
		TableMetaData table = getTableMetaData(con, tableName);
		if (table != null)
			return table.getPrimaryKeys();
		return null;
	}

	private void _initAdaptor()
	{
		String dbtype = info.getDbtype();
		String driver = info.getDriver();
		
		if (dbtype == null) {
			
			try {
				log.debug("Init DBAdapter from driver:" + driver);
				dbAdapter = DBFactory.create(driver);
			} catch (InstantiationException ex) {
				log.error(ex.getMessage());
			}
		} 
		else
		{
			try {
				log.debug("Init DBAdapter from dbtype:" + dbtype);
				dbAdapter = DBFactory.create(dbtype);
			} catch (InstantiationException ex1) {
				log.error(ex1.getMessage());
			}
		}
	}
	
	private String externalDBName = null;
	
	
	/**
	 * ʼݿ
	 */
	private void initDBAdapter() {
		if(!this.info.isExternal())
		{
			_initAdaptor();
		}
		else //ⲿԴʼdbadaptor
		{
			String external = this.info.getExternaljndiName();
			JDBCPool ep = null; 
			ep = DBUtil.getJDBCPoolByJNDIName(external,false);
			if(ep == null)
			{
				_initAdaptor();
			}
			else
			{
				externalDBName = ep.getDBName();
				
				{
					JDBCPool pool = SQLManager.getInstance().getPool(this.externalDBName,false);
					this.info.setExtenalInfo(pool.getJDBCPoolMetadata());
				}
			}
			
			
		}
		this.info.setDbtype(this.getDbAdapter().getDBTYPE());
	}
	
	private DBInfoEncrypt getDBInfoEncrypt()
	{
		try {
			Properties  p = BaseApplicationContext.fillProperties();
			if(p != null)
			{
				String DBInfoEncryptclass = p.getProperty("DBInfoEncryptclass");
				if(DBInfoEncryptclass != null && !DBInfoEncryptclass.trim().equals(""))
				{
					DBInfoEncryptclass = DBInfoEncryptclass.trim();
					return (DBInfoEncrypt)Class.forName(DBInfoEncryptclass).newInstance();
				}
				
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (Throwable e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
		}
		return new DESDBPasswordEncrypt();
	}
	private Properties getProperties()
	{
		Properties p = new Properties();
		p.setProperty(PoolManConstants.PROP_DRIVERCLASSNAME, info
				.getDriver());
		DBInfoEncrypt dbInfoEncrypt = getDBInfoEncrypt();
		if(this.info.isEncryptdbinfo())
		{
//			dbInfoEncrypt.decrypt(data)
			p.setProperty(PoolManConstants.PROP_URL, dbInfoEncrypt.decryptDBUrl(info.getURL()));
		}
		else 
		{
			p.setProperty(PoolManConstants.PROP_URL, info.getURL());
		}
		
		if (info.getPassword() != null)
		{
			if(this.info.isEncryptdbinfo())
			{
//				dbInfoEncrypt.decrypt(data)
				p.setProperty(PoolManConstants.PROP_PASSWORD, dbInfoEncrypt.decryptDBPassword(info
						.getPassword()));
			}
			else 
			{
				p.setProperty(PoolManConstants.PROP_PASSWORD, info
						.getPassword());
			}
			
			
		}
		if (info.getUserName() != null)
		{
			if(this.info.isEncryptdbinfo())
			{
				p.setProperty(PoolManConstants.PROP_USERNAME, dbInfoEncrypt.decryptDBUser(info
						.getUserName()));
			}
			else 
			{
				p.setProperty(PoolManConstants.PROP_USERNAME, info
						.getUserName());
			}
			
		}
		p.setProperty(PoolManConstants.PROP_MAXACTIVE, info
				.getMaximumSize()
				+ "");
		p.setProperty(PoolManConstants.PROP_INITIALSIZE, info
				.getInitialConnections()
				+ "");
		p.setProperty(PoolManConstants.PROP_MAXIDLE, info
				.getMaximumSize()
				+ "");
		p.setProperty(PoolManConstants.PROP_MINIDLE, info
				.getMinimumSize()
				+ "");

		p.setProperty(
				PoolManConstants.PROP_TIMEBETWEENEVICTIONRUNSMILLIS,
				info.getSkimmerFrequency() + "");
		p.setProperty(PoolManConstants.PROP_NUMTESTSPEREVICTIONRUN,
				info.getShrinkBy() + "");
		p.setProperty(PoolManConstants.PROP_MINEVICTABLEIDLETIMEMILLIS,
				(info.getConnectionTimeout()) + "");

		p.setProperty(PoolManConstants.PROP_MAXWAIT, info.getMaxWait());
		p.setProperty(PoolManConstants.PROP_REMOVEABANDONED, info
				.getRemoveAbandoned());
		p.setProperty(PoolManConstants.PROP_REMOVEABANDONEDTIMEOUT,
				info.getUserTimeout() + "");
		p.setProperty(PoolManConstants.PROP_TESTWHILEIDLE, info
				.isTestWhileidle()
				+ "");
		p.setProperty(PoolManConstants.PROP_TESTONBORROW, "true");
		if (info.getValidationQuery() != null
				&& !info.getValidationQuery().equals("")) {
			p.setProperty(PoolManConstants.PROP_VALIDATIONQUERY, info
					.getValidationQuery());
		}
		p.setProperty(PoolManConstants.PROP_LOGABANDONED, info
				.isLogAbandoned()
				+ "");
		p.setProperty(PoolManConstants.PROP_DEFAULTAUTOCOMMIT, "true");
		p.setProperty(PoolManConstants.PROP_DEFAULTREADONLY, info
				.isReadOnly()
				+ "");

		p.setProperty(
				PoolManConstants.PROP_DEFAULTTRANSACTIONISOLATION, info
						.getTxIsolationLevel()
						+ "");

		p.setProperty(PoolManConstants.PROP_POOLPREPAREDSTATEMENTS,
				info.isPoolPreparedStatements() + "");
		p.setProperty(PoolManConstants.PROP_MAXOPENPREPAREDSTATEMENTS,
				info.getMaxOpenPreparedStatements() + "");
		// if(info.getValidationQuery() != null)
		// p.setProperty(PoolManConstants.PROP_VALIDATIONQUERY,
		// info.getValidationQuery() );
		if (info.isMaximumSoft())
			p.setProperty(PoolManConstants.PROP_WHENEXHAUSTEDACTION,
					GenericObjectPool.WHEN_EXHAUSTED_GROW + "");
		else {
			p.setProperty(PoolManConstants.PROP_WHENEXHAUSTEDACTION,
					GenericObjectPool.WHEN_EXHAUSTED_BLOCK + "");
		}
		
		p.setProperty(PoolManConstants.PROP_USEPOOL, info.isUsepool() + "");
		// public final static String PROP_DEFAULTREADONLY =
		// "defaultReadOnly";
		// public final static String PROP_DEFAULTTRANSACTIONISOLATION =
		// "defaultTransactionIsolation";
		// public final static String PROP_DEFAULTCATALOG =
		// "defaultCatalog";
		// public final static String PROP_DRIVERCLASSNAME =
		// "driverClassName";
		// public final static String PROP_MAXACTIVE = "maxActive";
		// public final static String PROP_MAXIDLE = "maxIdle";
		// public final static String PROP_MINIDLE = "minIdle";
		// public final static String PROP_INITIALSIZE = "initialSize";
		// public final static String PROP_MAXWAIT = "maxWait";
		// public final static String PROP_TESTONBORROW =
		// "testOnBorrow";
		// public final static String PROP_TESTONRETURN =
		// "testOnReturn";
		// public final static String PROP_TIMEBETWEENEVICTIONRUNSMILLIS
		// = "timeBetweenEvictionRunsMillis";
		// public final static String PROP_NUMTESTSPEREVICTIONRUN =
		// "numTestsPerEvictionRun";
		// public final static String PROP_MINEVICTABLEIDLETIMEMILLIS =
		// "minEvictableIdleTimeMillis";
		// public final static String PROP_TESTWHILEIDLE =
		// "testWhileIdle";
		// public final static String PROP_PASSWORD = "password";
		// public final static String PROP_URL = "url";
		// public final static String PROP_USERNAME = "username";
		// public final static String PROP_VALIDATIONQUERY =
		// "validationQuery";
		// public final static String
		// PROP_ACCESSTOUNDERLYINGCONNECTIONALLOWED =
		// "accessToUnderlyingConnectionAllowed";
		// public final static String PROP_REMOVEABANDONED =
		// "removeAbandoned";
		// public final static String PROP_REMOVEABANDONEDTIMEOUT =
		// "removeAbandonedTimeout";
		// public final static String PROP_LOGABANDONED =
		// "logAbandoned";
		// public final static String PROP_POOLPREPAREDSTATEMENTS =
		// "poolPreparedStatements";
		// public final static String PROP_MAXOPENPREPAREDSTATEMENTS =
		// "maxOpenPreparedStatements";
		// public final static String PROP_CONNECTIONPROPERTIES =
		// "connectionProperties";
		//				 

		return p;
	}
	
	private void initPoolDatasource()
	{
		try {
			DataSource _datasource = null;
			if(StringUtil.isEmpty(this.info.getDatasourceFile()))//ûָIOCԴļֱӳʼԴapache dbcp
			{
				Properties p = getProperties();
				_datasource =  BasicDataSourceFactory
						.createDataSource(p);
			}
			else //iocļлȡԴʵ
			{
				_datasource =  DatasourceUtil.getDataSource(info.getDatasourceFile());
			}

			if (this.info.getJNDIName() != null
					&& !this.info.getJNDIName().equals("")) {
				this.datasource = new PoolManDataSource(_datasource, info
						.getDbname(), info.getJNDIName());
			} else {
				this.datasource = _datasource;
			}
			if(this.info.isEnablejta())
			{
				this.datasource = new TXDataSource(datasource);
			}
			
			

		} catch (Exception e) {
			e.printStackTrace();

		}
	}
	public void setUpCommonPool() {

		if (!this.info.isExternal()) {
			
				initPoolDatasource();
			
		} else {

			try {
				// Context ctx = new InitialContext();
				if(this.externalDBName == null)//õpoolman.xmlļжԴexternalDBNameΪգexternalDBNameΪձʾʾⲿԴʼԴⲿԴ
				{
					DataSource _datasource = find(this.info.getExternaljndiName(),info);
					
					if (_datasource != null) {
						
						if (this.info.getJNDIName() != null
								&& !this.info.getJNDIName().equals("")) {
							this.datasource = new PoolManDataSource(_datasource,
									info.getDbname(), info.getJNDIName());
						} else {
							this.datasource = _datasource;
						}
						
						if(this.info.isEnablejta())
						{
							if(!(_datasource instanceof TXDataSource))
							{
//								if(_datasource instanceof PoolManDataSource)
//								{
//									
//								}
//								else
								{
									this.datasource = new TXDataSource(datasource);
								}
							}
						}
					}
					
				}
			} catch (NamingException e) {
				log.info("ͨJNDI[" + info.getExternaljndiName()
						+ "]ȡⲿԴʧ:"+e.getMessage());
				// e.printStackTrace();
			}

		}

		

	}
	
	public DataSource find_(String jndiName) throws NamingException
	{
		
		if(ctx == null && this.dummyctx == null)
			return null;
		DataSource _datasource = ctx != null ?(DataSource) ctx.lookup(jndiName):null;
		if(_datasource != null)
			return _datasource;
		if(dummyctx != null)
			_datasource = (DataSource) dummyctx.lookup(jndiName);
		return _datasource;
	}

	public static DataSource find(String jndiName,JDBCPoolMetaData meta) throws NamingException {
		
			// Context ctx = new InitialContext();
			Map pools = SQLManager.getInstance().getPools();
			if(pools != null && pools.size() > 0)
			{
				Iterator its = pools.entrySet().iterator();
				String name = ContextUtil.handleJndiName(jndiName);
				Context ctx = null;
				while(its.hasNext())
				{
					try
					{
						Map.Entry ent = (Map.Entry)its.next();
						JDBCPool pool = (JDBCPool)ent.getValue();
						
						DataSource _datasource = (DataSource) pool.find_(name);
						if(_datasource != null)
							return _datasource;
					} catch (NamingException e) {
	
						continue;
						// e.printStackTrace();
					}
				}
				return null;
			}
			else
			{
				try {
					Context ctx = JDBCPool.buildContext(meta);
					DataSource _datasource = (DataSource) ctx.lookup(ContextUtil.handleJndiName(jndiName));
					return _datasource;
				} catch (NamingException e) {

					throw e;
					// e.printStackTrace();
				}
			}
		
	}
	
	
	public static DataSource find(String jndiName) throws NamingException {
		
		// Context ctx = new InitialContext();
		Map pools = SQLManager.getInstance().getPools();
		if(pools != null && pools.size() > 0)
		{
			Iterator its = pools.entrySet().iterator();
			String name = ContextUtil.handleJndiName(jndiName);
			Context ctx = null;
			while(its.hasNext())
			{
				try
				{
					Map.Entry ent = (Map.Entry)its.next();
					JDBCPool pool = (JDBCPool)ent.getValue();
					ctx = pool.ctx;
					if(ctx == null)
						continue;
					DataSource _datasource = (DataSource) ctx.lookup(name);
					if(_datasource != null)
						return _datasource;
				} catch (NamingException e) {

					continue;
					// e.printStackTrace();
				}
			}
			return null;
		}
		else
		{
			try {
				Context ctx = JDBCPool.buildContext(null);
				DataSource _datasource = (DataSource) ctx.lookup(ContextUtil.handleJndiName(jndiName));
				return _datasource;
			} catch (NamingException e) {

				throw e;
				// e.printStackTrace();
			}
		}
	
}

	public JDBCPool(JDBCPoolMetaData metad) {
		initctx(metad);
		
		this.deployedDataSource = false;
		this.info = metad;
		
		this.preparedStatementPool = new Hashtable();
		if (null == info.getJNDIName() || info.getJNDIName().equals(""))
			log
					.debug("JDBCPool["
							+ info.getDbname()
							+ "]: No JNDI name specified, The DataSource will not be binded to NamingContext.");
		else
			log.debug("JDBCPool[" + info.getDbname() + "]:INNER JNDI Name is "
					+ info.getJNDIName());
		if(this.info.getInterceptor() != null && !this.info.getInterceptor().equals("") &&  !this.info.getInterceptor().equals(defaultInterceptor_s))
		{
			try {
				this.interceptor = (InterceptorInf)Class.forName(this.info.getInterceptor()).newInstance();
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		startPool();

	}

	public String statusChecker() {
//		if(this.externalDBName == null)
			return status;
//		return SQLManager.getInstance().getPool(externalDBName).statusChecker();
		
	}

	
	public void startPool() {
		if (status.equals("start"))
			return;
		if (!status.equals("stop") && !status.equals("unknown"))
			return;
		initDBAdapter();
		
//		if(!this.isExternal())
		{
			if(this.datasource != null)
			{
				try {

					DatasourceUtil.closeDS(datasource);
				} catch (Exception e) {
					
				}
				datasource = null;
			}
		}

		this.setUpCommonPool();
		this.startTime = System.currentTimeMillis();
		this.status = "start";

		try {

			
			init();
			this.initDBProductInfo();
			log.debug("Load Database Meta Data=" + info.getLoadmetadata());
			if (info.getLoadmetadata() != null
					&& info.getLoadmetadata().equalsIgnoreCase("true") && !inited ) {
				log.debug("Load Database[" + info.getName()
						+ "] Meta Data beginning.....");
				if(this.externalDBName == null)
				{
					
					try {
						
						initDatabaseMetaData(null);
					} finally{
						inited = true;
					}
					
				}
				else
				{
					inited = true;
				}
				log.debug("Load Database[" + info.getName()
						+ "] Meta Data success.");
			} else {
				log.debug("Ignore Load Database[" + info.getName()
						+ "] Meta Data .");
			}
		} catch (Exception e) {
			log.debug("JDBCPool: Exception while initializing", e);
		}

	}
	
	private void initDBProductInfo()
	{
		
            Connection conn = null;
            try {
            	conn = requestConnection();;
                DatabaseMetaData md = conn.getMetaData();

                

                this.info.setDatabaseProductName(md.getDatabaseProductName());
                this.info.setDatabaseProductVersion(md.getDatabaseProductVersion());
                this.info.setDriverName(md.getDriverName());
                this.info.setDriverVersion(md.getDriverVersion());
                
                md = null;
                
            }catch(Exception e)
            {
            	
            }
            finally {
            	if(conn != null)
					try {
						conn.close();
					} catch (SQLException e) {
						e.printStackTrace();
					}
            }
	}

	/**
	 * ݿԪݷ˱仯װЩ
	 * 
	 * @param tableName
	 */
	public void updateTableMetaData(String tableName) {
		if(this.externalDBName == null)
		{
			Connection con = null;
	
			ResultSet rs = null;
			ResultSet columnrs = null;
			ResultSet primaryKeysrs = null;
			DatabaseMetaData metaData = null;
			ResultSet foreignrs = null;
	
			try {
				con = requestConnection();
	
				metaData = con.getMetaData();
//				String schemaName = this.getDbAdapter().getSchema(info);
//				if(schemaName == null)
//					schemaName = metaData.getUserName();
				String schemaName = getSchemaName_( metaData,this.getDbAdapter().getSchema(info));
				TableMetaData tableMetaData = null;
				// rs =
				// metaData.getTables(null,"sysmanager".toUpperCase(),"td_sm_job".toUpperCase(),new
				// String[] {"TABLE"});
				rs = metaData.getTables(null, schemaName, tableName
						.toUpperCase(), new String[] { "TABLE", "VIEW" });
				while (rs.next()) {
					tableName = rs.getString("TABLE_NAME");
	
					// System.out.println();
					// System.out.println("TABLE_SCHEM:"+rs.getString("TABLE_SCHEM"));
					// System.out.println("tableName:"+tableName);
					String tableType = rs.getString("TABLE_TYPE");
	
					tableMetaData = new TableMetaData();
					tableMetaData.setTableName(tableName);
					tableMetaData.setTableType(tableType);
					try {
						String tableRemark = rs.getString("REMARKS");
						tableRemark = this.getDbAdapter().getTableRemarks(con, tableName,
								tableRemark);
						tableMetaData.setRemarks(tableRemark);
					} catch (Exception e) {
	
					}
	
					// columnrs =
					// metaData.getColumns(null,"sysmanager".toUpperCase(),tableName,"%");
					columnrs = metaData.getColumns(null, schemaName,
							tableName, "%");
					while (columnrs.next()) {
						ColumnMetaData column = new ColumnMetaData(this.getDbAdapter());
						try {
							column.setColumnName(columnrs.getString("COLUMN_NAME"));
						} catch (Exception e) {
	
						}
						try {
							column.setColunmSize(columnrs.getInt("COLUMN_SIZE"));
						} catch (Exception e) {
	
						}
						try {
							column.setTypeName(columnrs.getString("TYPE_NAME"));
						} catch (Exception e) {
	
						}
						try {
							column.setIsNullable(columnrs.getString("IS_NULLABLE"));
						} catch (Exception e) {
	
						}
						try {
							column.setDECIMAL_DIGITS(columnrs
									.getInt("DECIMAL_DIGITS"));
						} catch (Exception e) {
	
						}
						try {
							column.setColumnDefaultValue(columnrs
									.getString("COLUMN_DEF"));
						} catch (Exception e) {
	
						}
						try {
							column.setNumPrecRadix(columnrs
									.getInt("NUM_PREC_RADIX"));
	
						} catch (Exception e) {
	
						}
						try {
							column.setDataType(columnrs.getInt("DATA_TYPE"));
						} catch (Exception e) {
	
						}
						try {
							column.setCHAR_OCTET_LENGTH(columnrs
									.getInt("CHAR_OCTET_LENGTH"));
						} catch (Exception e) {
	
						}
	
						try {
							String remarks_c = columnrs.getString("REMARKS");
							remarks_c = this.getDbAdapter().getColumnRemarks(con, tableName,
									column.getColumnName(), remarks_c);
	
							column.setRemarks(remarks_c);
						} catch (Exception e) {
	
						}
	
						tableMetaData.addColumns(column);
					}
					columnrs.close();
	
					/**
					 * Ϣ
					 */
					primaryKeysrs = metaData.getPrimaryKeys(null, schemaName, tableName);
					while (primaryKeysrs.next()) {
						PrimaryKeyMetaData primaryKeyMetaData = new PrimaryKeyMetaData(
								this.getDbAdapter());
						try {
							primaryKeyMetaData.setColumnName(primaryKeysrs
									.getString("COLUMN_NAME"));
						} catch (Exception e) {
	
						}
						try {
							primaryKeyMetaData.setPkName(primaryKeysrs
									.getString("PK_NAME"));
						} catch (Exception e) {
	
						}
						try {
							primaryKeyMetaData.setKeySEQ(primaryKeysrs
									.getInt("KEY_SEQ"));
						} catch (Exception e) {
	
						}
						// try
						// {
						// primaryKeyMetaData.setDataType(primaryKeysrs.getInt("DATA_TYPE"));
						// }
						// catch(Exception e)
						// {
						//						
						// }
						primaryKeyMetaData.setColumn(tableMetaData
								.getColumnMetaData(primaryKeyMetaData
										.getColumnName().toLowerCase()));
						tableMetaData.addPrimaryKey(primaryKeyMetaData);
	
					}
					primaryKeysrs.close();
					/**
					 * Ϣ
					 */
					foreignrs = metaData.getImportedKeys(null, schemaName, tableName);
					while (foreignrs.next()) {
						ForeignKeyMetaData foreignKeyMetaData = new ForeignKeyMetaData(
								this.getDbAdapter());
						try {
							foreignKeyMetaData.setPKTABLE_NAME(foreignrs
									.getString("PKTABLE_NAME"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setPKCOLUMN_NAME(foreignrs
									.getString("PKCOLUMN_NAME"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setFKTABLE_NAME(foreignrs
									.getString("FKTABLE_NAME"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setFKCOLUMN_NAME(foreignrs
									.getString("FKCOLUMN_NAME"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setColumnName(foreignrs
									.getString("FKCOLUMN_NAME"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setKEY_SEQ(foreignrs
									.getString("KEY_SEQ"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setUPDATE_RULE(foreignrs
									.getInt("UPDATE_RULE"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setDELETE_RULE(foreignrs
									.getInt("DELETE_RULE"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setFK_NAME(foreignrs
									.getString("FK_NAME"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setPK_NAME(foreignrs
									.getString("PK_NAME"));
						} catch (Exception e) {
	
						}
						try {
							foreignKeyMetaData.setDEFERRABILITY(foreignrs
									.getString("DEFERRABILITY"));
						} catch (Exception e) {
	
						}
						// try
						// {
						// foreignKeyMetaData.setDataType(foreignrs.getInt("DATA_TYPE"));
						// }
						// catch(Exception e)
						// {
						//						
						// }
						foreignKeyMetaData.setColumn(tableMetaData
								.getColumnMetaData(foreignKeyMetaData
										.getColumnName().toLowerCase()));
						tableMetaData.addForeignKey(foreignKeyMetaData);
	
					}
	
					foreignrs.close();
					//
					// ResultSet rs4 =
					// metaData.getExportedKeys(null,null,tableName);
	
					// this.tableMetaData.put(rsMeta.)
					this.tableMetaDatasindexByTablename.put(
							tableName.toLowerCase(), tableMetaData);
					tableMetaDatas.add(tableMetaData);
				}
				// System.out.println("this.tableMetaDatasindexByTablename.put(tableName.toLowerCase(),tableMetaData):"
				// + tableName + "=" + tableMetaData);
				rs.close();
				// metaData.
			} catch (SQLException e) {
				// TODO Auto-generated catch block
	
			} catch (Exception e) {
				// e.printStackTrace();
			}
	
			finally {
				try {
					if (rs != null)
						rs.close();
				} catch (Exception e) {
	
				}
				try {
					if (columnrs != null)
						columnrs.close();
				} catch (Exception e) {
	
				}
				try {
					if (primaryKeysrs != null)
						primaryKeysrs.close();
				} catch (Exception e) {
	
				}
				try {
					if (foreignrs != null)
						foreignrs.close();
				} catch (Exception e) {
	
				}
				try {
					if (con != null)
						con.close();
				} catch (Exception e) {
	
				}
	
			}
		}
		else
		{
			SQLManager.getInstance().getPool(externalDBName).updateTableMetaData(tableName);
		}
	}

	/**
	 * ʼݿԪ
	 * 
	 */
	private void initDatabaseMetaData(Connection con) {
		

		ResultSet rs = null;
		
		DatabaseMetaData metaData = null;
		

		try {
			if(con == null)
				con = requestConnection();

			metaData = con.getMetaData();
			String schemaName = getSchemaName_( metaData,this.getDbAdapter().getSchema(info));
//			String schemaName = this.getDbAdapter().getSchema(info);
//			if(schemaName == null)
//				schemaName = metaData.getUserName();
			// rs =
			// metaData.getTables(null,"sysmanager".toUpperCase(),"td_sm_job".toUpperCase(),new
			// String[] {"TABLE"});
			rs = metaData.getTables(null, schemaName, "%",
					new String[] { "TABLE", "VIEW" });
			while (rs.next()) {
				String tableName = rs.getString("TABLE_NAME");
				if (tableName.startsWith("BIN$"))
					continue;
				if (tableMetaDatasindexByTablename.containsKey(tableName
						.toLowerCase())) {
					System.out.println("table [" + tableName + "] Ѿأԣ");
					continue;
				}
				log.debug("load table[" + tableName + "]'s metadata.");
				// System.out.println();
				// System.out.println("TABLE_SCHEM:"+rs.getString("TABLE_SCHEM"));
				// System.out.println("tableName:"+tableName);
				TableMetaData tableMetaData = buildTableMetaData(con,rs,metaData ,true);
				if(tableMetaData == null)
					continue;
				//
				// ResultSet rs4 =
				// metaData.getExportedKeys(null,null,tableName);

				// this.tableMetaData.put(rsMeta.)
				this.tableMetaDatasindexByTablename.put(
						tableName.toLowerCase(), tableMetaData);
				tableMetaDatas.add(tableMetaData);
			}
			rs.close();

			// metaData.
		} catch (SQLException e) {
			e.printStackTrace();
			// TODO Auto-generated catch block

		} catch (Exception e) {
			e.printStackTrace();
		}

		finally {
			try {
				if (rs != null)
					rs.close();
			} catch (Exception e) {

			}
			
			try {
				if (con != null)
					con.close();
			} catch (Exception e) {

			}

		}

	}

	/**
	 * ʼݿԪ added by biaoping.yin on 20080529
	 * ȡıԪݽֱˢµ
	 */
	public TableMetaData getTableMetaDataFromDatabase(
			String tableName)
	{
		return getTableMetaDataFromDatabase(null,
				tableName);
	}
	
	
	
	private String[] convertTableTypes(String tableTypes[])
	{
		if(tableTypes == null || tableTypes.length == 0)
			return new String[] { "TABLE", "VIEW" };
		
		for(int i = 0; i < tableTypes.length; i ++)
		{
			if(tableTypes[i].equals(JDBCPool.TABLE_TYPE_ALL))
				return new String[] { "TABLE", "VIEW" };
			
			
		}
		
		return tableTypes;
	}
	/**
	 * ȡϢ
	 */
	public List<TableMetaData> getTablesFromDatabase(String tableNamePattern)
	{
		return getTablesFromDatabase(null,tableNamePattern);
	}
	
	/**
	 * ȡϢ
	 */
	public List<TableMetaData> getTablesFromDatabase(String tableNamePattern,String tableTypes[])
	{
		return getTablesFromDatabase(null,tableNamePattern,tableTypes);
	}
	
	public List<TableMetaData> getTablesFromDatabase(Connection con) {
		return getTablesFromDatabase(con,(String)null);
	}
	
	public List<TableMetaData> getTablesFromDatabase(Connection con,String tabletypes[]) {
		return getTablesFromDatabase(con,(String)null, tabletypes);
	}
	
	
	public List<TableMetaData> getTablesFromDatabase(Connection con,String tableNamePattern)
	{
		return getTablesFromDatabase(con,tableNamePattern,null);
	}
	/**
	 * ȡϢ
	 * @param con
	 * @param tableName
	 * @return
	 */
	public List<TableMetaData> getTablesFromDatabase(Connection con,String tableNamePattern,String tableTypes[]) {
		return getTablesFromDatabase(con,tableNamePattern,tableTypes,false);
	}
	/**
	 * ȡϢ
	 */
	public List<TableMetaData> getTablesFromDatabase()
	{
		return getTablesFromDatabase((Connection)null);
	}
	
	/**
	 * ȡϢ
	 */
	public List<TableMetaData> getTablesFromDatabase(String tabletypes[])
	{
		return getTablesFromDatabase((Connection)null,tabletypes);
	}
	
	
	
	/**
	 * ȡϢ
	 */
	public List<TableMetaData> getTablesFromDatabase(String tableNamePattern,boolean loadColumns)
	{
		return getTablesFromDatabase(null,tableNamePattern,loadColumns);
	}
	
	/**
	 * ȡϢ
	 */
	public List<TableMetaData> getTablesFromDatabase(String tableNamePattern,String tableTypes[],boolean loadColumns)
	{
		return getTablesFromDatabase(null,tableNamePattern,tableTypes,loadColumns);
	}
	
	public List<TableMetaData> getTablesFromDatabase(Connection con,boolean loadColumns) {
		return getTablesFromDatabase(con,(String)null,loadColumns);
	}
	
	public List<TableMetaData> getTablesFromDatabase(Connection con,String tabletypes[],boolean loadColumns) {
		return getTablesFromDatabase(con,(String)null, tabletypes,loadColumns);
	}
	
	
	public List<TableMetaData> getTablesFromDatabase(Connection con,String tableNamePattern,boolean loadColumns)
	{
		return getTablesFromDatabase(con,tableNamePattern,null,loadColumns);
	}
	/**
	 * ȡϢ
	 * @param con
	 * @param tableName
	 * @return
	 */
	public List<TableMetaData> getTablesFromDatabase(Connection con,String tableNamePattern,String tableTypes[],boolean loadColumns) {
		List<TableMetaData> tableMetaDatas = new ArrayList<TableMetaData>();
		ResultSet rs = null;
		DatabaseMetaData metaData = null;
		boolean outcon = true;
		
		JDBCTransaction tx = null;
		try {
			if (con == null) {
				tx = TransactionManager.getTransaction();
				if (tx == null)
					con = requestConnection();
				else
					con = tx.getConnection(this.getDBName());
				outcon = false;
			}

			metaData = con.getMetaData();
			String schemaName = getSchemaName_( metaData,this.getDbAdapter().getSchema(info));
//			String schemaName = this.getDbAdapter().getSchema(info);
//			if(schemaName == null)
//				schemaName = metaData.getUserName();

			// rs =
			// metaData.getTables(null,"sysmanager".toUpperCase(),"td_sm_job".toUpperCase(),new
			// String[] {"TABLE"});
			String temp = this.getDbAdapter()
			.getSchemaTableTableName(info, tableNamePattern);
			rs = metaData.getTables(null, schemaName, temp == null?"%":temp,
					this.convertTableTypes(tableTypes)
					);
			while (rs.next()) {
				TableMetaData tableMetaData = buildTableMetaData(con,rs,metaData ,loadColumns);
				if(tableMetaData == null)
					continue;
				//
				// ResultSet rs4 =
				// metaData.getExportedKeys(null,null,tableName);

				// this.tableMetaData.put(rsMeta.)
				
				tableMetaDatas.add(tableMetaData);
			}
			rs.close();

			// metaData.
		} catch (SQLException e) {
			e.printStackTrace();
			// TODO Auto-generated catch block

		} catch (Exception e) {
			e.printStackTrace();
		}

		finally {
			try {
				if (rs != null)
					rs.close();
			} catch (Exception e) {

			}
			
			try {
				if (con != null) {
					if (!outcon && tx == null)
						con.close();
				}
			} catch (Exception e) {

			}
			con = null;

		}
		return tableMetaDatas;
	}
	/**
	 * ȡϢ
	 */
	public List<TableMetaData> getTablesFromDatabase(boolean loadColumns)
	{
		return getTablesFromDatabase((Connection)null, loadColumns);
	}
	
	/**
	 * ȡϢ
	 */
	public List<TableMetaData> getTablesFromDatabase(String tabletypes[],boolean loadColumns)
	{
		return getTablesFromDatabase((Connection)null,tabletypes,loadColumns);
	}
	
	
	
	

	private TableMetaData buildTableMetaData(Connection con,ResultSet rs,DatabaseMetaData metaData ,boolean loadColumns) throws SQLException
	{
		ResultSet columnrs = null;
		ResultSet primaryKeysrs = null;
		
		ResultSet foreignrs = null;
		String tableName = rs.getString("TABLE_NAME");
		if (tableName.startsWith("BIN$"))
			return null;
		log.debug("load table[" + tableName + "]'s metadata.");

		String tableType = rs.getString("TABLE_TYPE");

		TableMetaData tableMetaData = new TableMetaData();
		tableMetaData.setTableName(tableName);
		tableMetaData.setTableType(tableType);
		try {
			String tableRemark = rs.getString("REMARKS");

			tableRemark = this.getDbAdapter().getTableRemarks(con, tableName,
					tableRemark);
			tableMetaData.setRemarks(tableRemark);
		} catch (Exception e) {

		}
		String schemaName = getSchemaName_( metaData,this.getDbAdapter().getSchema(info));
//		String schemaName = this.getDbAdapter().getSchema(info);
//		if(schemaName == null)
//			schemaName = metaData.getUserName();
		if(loadColumns)
		{
			
			try {
				
				columnrs = metaData.getColumns(null, schemaName, tableName, "%");
	
				while (columnrs.next()) {
					ColumnMetaData column = new ColumnMetaData(
							this.getDbAdapter());
					try {
						column.setColumnName(columnrs
								.getString("COLUMN_NAME"));
					} catch (Exception e) {
	
					}
					try {
						column
								.setColunmSize(columnrs
										.getInt("COLUMN_SIZE"));
					} catch (Exception e) {
	
					}
					try {
						column.setTypeName(columnrs.getString("TYPE_NAME"));
					} catch (Exception e) {
	
					}
					try {
						column.setIsNullable(columnrs
								.getString("IS_NULLABLE"));
					} catch (Exception e) {
	
					}
					try {
						column.setColumnDefaultValue(columnrs
								.getString("COLUMN_DEF"));
					} catch (Exception e) {
	
					}
					try {
						column.setNumPrecRadix(columnrs
								.getInt("NUM_PREC_RADIX"));
	
					} catch (Exception e) {
	
					}
					try {
						column.setDataType(columnrs.getInt("DATA_TYPE"));
					} catch (Exception e) {
	
					}
					try {
						column.setDECIMAL_DIGITS(columnrs
								.getInt("DECIMAL_DIGITS"));
					} catch (Exception e) {
	
					}
					try {
						column.setNumPrecRadix(columnrs
								.getInt("NUM_PREC_RADIX"));
	
					} catch (Exception e) {
	
					}
	
					try {
						String remarks_c = columnrs.getString("REMARKS");
						remarks_c = this.getDbAdapter().getColumnRemarks(con,
								tableName, column.getColumnName(),
								remarks_c);
	
						column.setRemarks(remarks_c);
	
					} catch (Exception e) {
	
					}
	
					tableMetaData.addColumns(column);
				}
				columnrs.close();
			} catch (Exception e) {
				columnrs.close();
				columnrs = null;
				e.printStackTrace();
			}
	
			/**
			 * Ϣ
			 */
			try {
				primaryKeysrs = metaData.getPrimaryKeys(null, schemaName, tableName);
	
				while (primaryKeysrs.next()) {
					PrimaryKeyMetaData primaryKeyMetaData = new PrimaryKeyMetaData(
							this.getDbAdapter());
					try {
						primaryKeyMetaData.setColumnName(primaryKeysrs
								.getString("COLUMN_NAME"));
					} catch (Exception e) {
	
					}
					try {
						primaryKeyMetaData.setPkName(primaryKeysrs
								.getString("PK_NAME"));
					} catch (Exception e) {
	
					}
					try {
						primaryKeyMetaData.setKeySEQ(primaryKeysrs
								.getInt("KEY_SEQ"));
					} catch (Exception e) {
	
					}
					// try
					// {
					// primaryKeyMetaData.setDataType(primaryKeysrs.getInt("DATA_TYPE"));
					// }
					// catch(Exception e)
					// {
					//						
					// }
					primaryKeyMetaData.setColumn(tableMetaData
							.getColumnMetaData(primaryKeyMetaData
									.getColumnName().toLowerCase()));
					tableMetaData.addPrimaryKey(primaryKeyMetaData);
	
				}
				primaryKeysrs.close();
				primaryKeysrs = null;
			} catch (Exception e) {
				primaryKeysrs.close();
				primaryKeysrs = null;
				e.printStackTrace();
			}
	
			try {
				/**
				 * Ϣ
				 */
				foreignrs = metaData.getImportedKeys(null, schemaName, tableName);
				while (foreignrs.next()) {
					ForeignKeyMetaData foreignKeyMetaData = new ForeignKeyMetaData(
							this.getDbAdapter());
					try {
						foreignKeyMetaData.setPKTABLE_NAME(foreignrs
								.getString("PKTABLE_NAME"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setPKCOLUMN_NAME(foreignrs
								.getString("PKCOLUMN_NAME"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setFKTABLE_NAME(foreignrs
								.getString("FKTABLE_NAME"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setFKCOLUMN_NAME(foreignrs
								.getString("FKCOLUMN_NAME"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setColumnName(foreignrs
								.getString("FKCOLUMN_NAME"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setKEY_SEQ(foreignrs
								.getString("KEY_SEQ"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setUPDATE_RULE(foreignrs
								.getInt("UPDATE_RULE"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setDELETE_RULE(foreignrs
								.getInt("DELETE_RULE"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setFK_NAME(foreignrs
								.getString("FK_NAME"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setPK_NAME(foreignrs
								.getString("PK_NAME"));
					} catch (Exception e) {
	
					}
					try {
						foreignKeyMetaData.setDEFERRABILITY(foreignrs
								.getString("DEFERRABILITY"));
					} catch (Exception e) {
	
					}
	
					foreignKeyMetaData.setColumn(tableMetaData
							.getColumnMetaData(foreignKeyMetaData
									.getColumnName().toLowerCase()));
					tableMetaData.addForeignKey(foreignKeyMetaData);
	
				}
				foreignrs.close();
			} catch (Exception e) {
	
				foreignrs.close();
				foreignrs = null;
				e.printStackTrace();
			}
		}
		return tableMetaData;
	}
	
	private String getSchemaName_(DatabaseMetaData metaData,String schemaName) throws SQLException
	{
		if(schemaName == null)
			schemaName = metaData.getUserName();
		else if(schemaName == DB.NULL_SCHEMA)
			schemaName = null;
		return schemaName;
	}
	/**
	 * ʼݿԪ added by biaoping.yin on 20080529
	 * ȡıԪݽֱˢµ
	 */
	public TableMetaData getTableMetaDataFromDatabase(Connection con,
			String tableName) {

		
		if(this.externalDBName == null)
		{
			
			ResultSet rs = null;
			
			DatabaseMetaData metaData = null;
			
			TableMetaData tableMetaData = null;
			boolean outcon = true;
	
			JDBCTransaction tx = null;
			try {
				if (con == null) {
					tx = TransactionManager.getTransaction();
					if (tx == null)
						con = requestConnection();
					else
						con = tx.getConnection(this.getDBName());
					outcon = false;
				}
	
				metaData = con.getMetaData();
				String schemaName = getSchemaName_( metaData,this.getDbAdapter().getSchema(info));
//				String schemaName = this.getDbAdapter().getSchema(info);
//				if(schemaName == null)
//					schemaName = metaData.getUserName();
				// rs =
				// metaData.getTables(null,"sysmanager".toUpperCase(),"td_sm_job".toUpperCase(),new
				// String[] {"TABLE"});
				rs = metaData.getTables(null, schemaName, this.getDbAdapter()
						.getSchemaTableTableName(info, tableName), new String[] {
						"TABLE", "VIEW" });
				while (rs.next()) {
					tableMetaData = buildTableMetaData(con,rs,metaData ,true);
					if(tableMetaData == null)
						continue;
					//
					// ResultSet rs4 =
					// metaData.getExportedKeys(null,null,tableName);
	
					// this.tableMetaData.put(rsMeta.)
					this.tableMetaDatasindexByTablename.put(
							tableMetaData.getTableName().toLowerCase(), tableMetaData);
					tableMetaDatas.add(tableMetaData);
	
				}
				rs.close();
	
				// metaData.
			} catch (SQLException e) {
				e.printStackTrace();
				// TODO Auto-generated catch block
	
			} catch (Exception e) {
				e.printStackTrace();
			}
	
			finally {
				try {
					if (rs != null)
						rs.close();
				} catch (Exception e) {
	
				}
				
				try {
					if (con != null) {
						if (!outcon && tx == null)
							con.close();
					}
				} catch (Exception e) {
	
				}
				con = null;
	
			}
			return tableMetaData;
		}
		else
		{
			return SQLManager.getInstance().getPool(externalDBName).getTableMetaDataFromDatabase(con, tableName);
		}

	}

	public void init() throws Exception {


		deployDataSource();

	}

	/** Associates a DataSource view with this pool. */
	public void setDataSource(PoolManDataSource ds) {
		this.datasource = ds;
	}

	public DataSource getDataSource() {
		if (this.datasource == null) {
			log.info(info.getDbname() + " has no associated DataSource.");
			throw new NullPointerException(info.getDbname()
					+ " has no associated DataSource.");
		}
		return this.datasource;
	}

	public void deployDataSource() {

		if (this.datasource == null || this.info.getJNDIName() == null
				|| this.info.getJNDIName().equals(""))
			return;

		try {
			log.debug("Bind the DataSource view of this pool to JNDI");
			String jndiName = this.info.getJNDIName();
			
			
			if (!exist(ctx, jndiName,info)) {
				bind(ctx, jndiName, this.datasource,info);
			}

			this.deployedDataSource = true;

			log.info("DataSource bound to JNDI under name[" + jndiName + "]");
			

		} catch (Exception e) {
			
			log.info(e.getMessage());
		}
	}

	/**
	 * Bind val to name in ctx, and make sure that all intermediate contexts
	 * exist.
	 * 
	 * @param ctx
	 *            the root context
	 * @param name
	 *            the name as a string
	 * @param val
	 *            the object to be bound
	 * @throws NamingException
	 */
	public void bind(Context ctx, String name, Object val,JDBCPoolMetaData meta)
			throws NamingException {
		try {
			initctx(meta);			
			ctx.rebind(name, val);
			log.debug("binding datasource  to container context  with: " + name);
			
		} catch (Exception e) {
			Name n = ctx.getNameParser("").parse(name);
			while (n.size() > 1) {
				String ctxName = n.get(0);

				Context subctx = null;
				try {
					log.debug("lookup: " + ctxName);
					subctx = (Context) ctx.lookup(ctxName);
				} catch (NameNotFoundException nfe) {
				}

				if (subctx != null) {
					log.debug("Found subcontext: " + ctxName);
					ctx = subctx;
				} else {
					log.info("Creating subcontext: " + ctxName);
					ctx = ctx.createSubcontext(ctxName);
				}
				n = n.getSuffix(1);
			}
			log.debug("binding: " + n);
			ctx.rebind(n, val);
		}
		finally
		{
			this.dummyctx.rebind(name, val);
			log.debug("Bound name " + name + " to dummy context.");
		}
		
	}

	/**
	 * жjndiNameǷ
	 * 
	 * @param ctx
	 * @param jndiName
	 * @return
	 */
	private boolean exist(Context ctx, String jndiName,JDBCPoolMetaData meta) {
		try {
			initctx(meta);
			ctx = this.ctx;
			DataSource datasource = (DataSource) find_(ContextUtil.handleJndiName(jndiName));
			if (datasource == null) {

				return false;
			} else {
				System.out.println("DataSource bound to JNDI under name["
						+ jndiName + "]");
				return true;
			}
		} catch (Exception e) {
			// e.printStackTrace();
			return false;
		}
	}

	public void undeployDataSource() throws Exception {

		if (this.datasource == null || this.info.getJNDIName() == null
				|| this.info.getJNDIName().equals(""))
			return;

		try {
			try {

				DatasourceUtil.closeDS(datasource);
			} catch (Exception e) {
				throw e;
			}
			try {
				
				if(ctx != null)
					ctx.unbind(this.info.getJNDIName());
			} catch (Exception e) {

			}
			this.deployedDataSource = false;
			
		} catch (Exception e) {
			throw e;
		}
	}

	public boolean isDataSourceDeployed() {
		return this.deployedDataSource;
	}


	/**
	 * Determine whether or not this pool is using a SQLCache (configured in
	 * poolman.xml and disabled by default).
	 */
	public boolean usingCache() {
		// if (this.sqlcache == null)
		// return false;
		return true;
	}

	/** Force the cache to refresh. */
	public void refreshCache() {

	}

	public String getDriver() {

		return info.getDriver();
	}

	public String getURL() {
		if(this.externalDBName == null)
			return info.getURL();
		JDBCPoolMetaData meta = SQLManager.getInstance().getJDBCPoolMetaData(externalDBName);
		if(meta != null)
			return meta.getURL();
		return info.getURL();
		
	}

	public String getUserName() {
		if(this.externalDBName == null)
			return info.getUserName();
		JDBCPoolMetaData meta = SQLManager.getInstance().getJDBCPoolMetaData(externalDBName);
		if(meta != null)
			return meta.getUserName();
		return info.getUserName();
		
	}

	public String getPassword() {
		if(this.externalDBName == null)
			return info.getPassword();
		JDBCPoolMetaData meta = SQLManager.getInstance().getJDBCPoolMetaData(externalDBName);
		if(meta != null)
			return meta.getPassword();
		return info.getPassword();
		
	}

	public int getTransactionIsolation() {
		if(this.externalDBName == null)
			return info.getIsolationLevel();
		JDBCPoolMetaData meta = SQLManager.getInstance().getJDBCPoolMetaData(externalDBName);
		if(meta != null)
			return meta.getIsolationLevel();
		return info.getIsolationLevel();
		
	}

	/**
	 * added by biaoping.yin on 20050601 ȡݿģʽ
	 * 
	 * @return String
	 */
	public String getKeygenerate() {
		if(this.externalDBName == null)
			return info.getKeygenerate();
		JDBCPoolMetaData meta = SQLManager.getInstance().getJDBCPoolMetaData(externalDBName);
		if(meta != null)
			return meta.getKeygenerate();
		return info.getKeygenerate();
		
	}

	/**
	 * added by biaoping.yin on 20050601 ȡݿ
	 * 
	 * @return String
	 */
	public String getDBType() {
//		return info.getDbtype();
		return this.getDbAdapter().getDBTYPE();
	}

	public boolean isUsingNativeResults() {

			return info.isNativeResults();

		
	}

	public void checkCredentials(String username, String password)
			throws SQLException {
		if(this.externalDBName == null)
		{
			if ((this.info.getUserName().equals(username))
					&& (this.info.getPassword().equals(password))) {
				log
						.debug("Invalid Username/Password: " + username + "/"
								+ password);
				throw new SQLException("Invalid Username/Password: " + username
						+ "/" + password);
			}
		}
		else
		{
			SQLManager.getInstance().getPool(externalDBName).checkCredentials(username, password);
		}
	}



	/**
	 * Retrieves a PooledConnection impl and returns its Handle. 
	 * ֱӴԭʼлȡconnectiondatasource һTXDataSource
	 * ôҪTXDataSourceڲԭʼԴȻлȡӦconnection
	 */
	public Connection requestConnection() throws SQLException {
		if(datasource != null)
		{
			if(!(datasource instanceof TXDataSource))
				return this.datasource.getConnection();
			else
			{
				return ((TXDataSource)this.datasource).getSRCDataSource().getConnection();
			}
		}
		else
		{
			if(this.isExternal() )
			{
				if(this.externalDBName == null)
				{
					throw new NestedSQLException("Request Connection failed:DB Pool[dbname=" + this.info.getDbname() + ",jndiname="+ this.info.getJNDIName() +",extenerjndiname="+ this.info.getExternaljndiName() +"] is stopped or not inited. Please restarted the pool.");
				}
				else
				{
					return SQLManager.getInstance().getPool(externalDBName).requestConnection();
				}
			
			}
			else
			{
				throw new NestedSQLException("Request Connection failed:DB Pool[" + this.info.getDbname() + "] is stopped. Please restarted the pool.");	
			}
		}
	
	}



	/**
	 * ȡǰӳʹõ ӿֻڲԴãⲿԴ-1
	 * 
	 * @return
	 */
	public int getNumActive() {
		if(this.externalDBName == null)
		{

			return DatasourceUtil.getNumActive(datasource);
		}
		else
		{
			return SQLManager.getInstance().getPool(externalDBName).getNumActive();
		}
	}
	
	/**
	 * ȡǰӳʹõ ӿֻڲԴãⲿԴ-1
	 * ӳԴΪʵָù
	 * @deprecated ʹñӿ
	 * @return
	 */
	public List getTraceObjects() {
		if(this.externalDBName == null)
		{
			return DatasourceUtil.getTraceObjects(datasource);
		}
		else
		{
			return SQLManager.getInstance().getPool(externalDBName).getTraceObjects();
		}
	}
	

	private DataSource getSRCDataSource(TXDataSource ds)
	{
		return ds.getSRCDataSource();
	}

	/**
	 * ȡʹ¼ӳصĿǰΪֹʹӵĿ ⲿԴ-1
	 * 
	 * @return
	 */
	public int getMaxNumActive() {
		if(this.externalDBName == null)
		{

			return DatasourceUtil.getMaxNumActive(datasource);
		}
		else
		{
			return SQLManager.getInstance().getPool(externalDBName).getMaxNumActive();
		}
	}

	/**
	 * ȡǰӳпе ӿֻڲԴãⲿԴ-1
	 * ӳԴΪʵָù
	 * @return
	 */
	public int getNumIdle() {
		if(this.externalDBName == null)
		{
			

			return DatasourceUtil.getNumIdle(datasource);
		}
		else
		{
			return SQLManager.getInstance().getPool(externalDBName).getNumIdle();
		}
	}

	public void stopPool() throws Exception {

		closeAllResources();
	}

	/**
	 * Overriden in order to ensure that JNDI resources are disposed of
	 * properly.
	 * @throws Exception 
	 */
	public void closeAllResources() throws Exception {
		if (this.status.equals("stop"))
			return;
		if (!this.status.equals("start"))
			return;
		
		System.out.println("Shutdown poolman[" + this.getDBName() + "] start.");
		log.debug("Shutdown poolman[" + this.getDBName() + "] start.");
			
			undeployDataSource();
			this.stopTime = System.currentTimeMillis();
			this.status = "stop";
			if (this.tableMetaDatas != null) {
				try {
					tableMetaDatas.clear();
					tableMetaDatas = new TreeSet();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			if(this.preparedStatementPool != null)
				this.preparedStatementPool.clear();
			if(tableMetaDatasindexByTablename != null)
				this.tableMetaDatasindexByTablename.clear();
			this.datasource = null;
			this.inited = false;
	
		System.out.println("Shutdown poolman[" + this.getDBName() + "] ok.");

		
	}

	/**
	 * Static method that closes the Connection, Statement and ResultSets in one
	 * place.
	 */
	public static void closeResources(Connection con, Statement statement,
			ResultSet resultset) {
		closeResultSet(resultset);
		closeStatement(statement);
		closeConnection(con);
	}

	/**
	 * Static method that closes the statement and result sets in one place;
	 * this is here as a convenience to shorten the finally block in statements.
	 * Both arguments may be null;
	 * 
	 * @param statement
	 *            the statement to be closed
	 * @param resultSet
	 *            the resultSet to be closed
	 */
	public static void closeResources(Statement statement, ResultSet resultSet) {

		closeResultSet(resultSet);
		closeStatement(statement);
	}

	public static void closeConnection(Connection con) {
		try {
			if (con != null && !con.isClosed())
				con.close();
		} catch (SQLException e) {
		}
		con = null;
	}

	/**
	 * Closes the given statement. It is here to get rid of the extra try block
	 * in finally blocks that need to close statements
	 * 
	 * @param statement
	 *            the statement to be closed. may be null
	 */
	public static void closeStatement(Statement statement) {
		try {
			if (statement != null) {
				statement.close();
			}
		} catch (SQLException e) {
		}
		statement = null;
	}

	/**
	 * This method closes the given ResultSet.
	 * 
	 * @param rs
	 *            the ResultSet to be closed. May be null.
	 */
	public static void closeResultSet(ResultSet rs) {
		try {
			if (rs != null) {
				rs.close();
			}
		} catch (SQLException e) {
		}
		// //added by biaoping.yin on 2005.03.28
		catch (Exception e) {
			// System.out.println("");
		}
		rs = null;
	}

	/**
	 * ȡݿ
	 * 
	 * @return DB
	 */
	public DB getDbAdapter() {
		if(this.externalDBName == null)
		{
			return dbAdapter;
			
		}
		else
		{
			return SQLManager.getInstance().getDBAdapter(externalDBName);
		}
			
	}

	public String getDBName() {
		return info.getDbname();
	}

	private boolean inited = false;

	public Set getTableMetaDatas() {

		if(this.externalDBName == null)
		{
			String load = this.info.getLoadmetadata();
			if (load.equalsIgnoreCase("false") && !inited) {
				synchronized (tableMetaDatas) {
					if (!inited) {
	
						try {
							this.initDatabaseMetaData(null);
							inited = true;
	
						} catch (Exception e) {
							inited = true;
	
						}
	
						// return tableMetaDatas;
	
					}
				}
				// return tableMetaDatas;
			}
			return tableMetaDatas;
		}
		else
		{
			return SQLManager.getInstance().getPool(externalDBName).getTableMetaDatas();
		}
	}

	public boolean isRobotQuery() {
//		if(this.externalDBName == null)
		{
			return this.info.isRobotquery();
		}
//		else
//		{
//			return SQLManager.getInstance().getPool(externalDBName).isRobotQuery();
//		}
	}

	public JDBCPoolMetaData getJDBCPoolMetadata() {
		return info;
	}

	public String getStatus() {
		if(this.externalDBName == null)
			return status;
		return SQLManager.getInstance().getPool(externalDBName).getStatus();
		
	}

	public void setStatus(String status) {
		this.status = status;
	}

	public long getStartTime() {
		if(this.externalDBName == null)
			return startTime;
		return SQLManager.getInstance().getPool(externalDBName).getStartTime();
	}

	public void setStartTime(long startTime) {
		this.startTime = startTime;
	}

	public long getStopTime() {
		if(this.externalDBName == null)
			return stopTime;
		return SQLManager.getInstance().getPool(externalDBName).getStopTime();
	}

	/**
	 * @return the interceptor
	 */
	public InterceptorInf getInterceptor() {
		return interceptor;
	}

	/**
	 * @param interceptor the interceptor to set
	 */
	public void setInterceptor(InterceptorInf interceptor) {
		this.interceptor = interceptor;
	}


	public boolean isExternal()
	{
		return getJDBCPoolMetadata().isExternal();
	}

	/**
	 * @return the externalDBName
	 */
	public String getExternalDBName() {
		return externalDBName;
	}
	
	public boolean isAutoprimarykey() {

			return this.info.getAutoprimarykey();

	}
	
	public boolean showsql()
	{
		return this.info.isShowsql();
	}
}
